@@ -70,6 +70,7 @@ from _drgn import (
     StackTrace,
     Symbol,
     SymbolBinding,
+    SymbolIndex,
     SymbolKind,
     Thread,
     Type,
@@ -79,11 +80,13 @@ from _drgn import (
     TypeMember,
     TypeParameter,
     TypeTemplateParameter,
+    alignof,
     cast,
     container_of,
     filename_matches,
     get_default_prog,
     host_platform,
+    implicit_convert,
     offsetof,
     program_from_core_dump,
     program_from_kernel,
@@ -125,6 +128,7 @@ __all__ = (
     "StackTrace",
     "Symbol",
     "SymbolBinding",
+    "SymbolIndex",
     "SymbolKind",
     "Thread",
     "Type",
@@ -134,12 +138,14 @@ __all__ = (
     "TypeMember",
     "TypeParameter",
     "TypeTemplateParameter",
+    "alignof",
     "cast",
     "container_of",
     "execscript",
     "filename_matches",
     "get_default_prog",
     "host_platform",
+    "implicit_convert",
     "offsetof",
     "program_from_core_dump",
     "program_from_kernel",
