@@ -11,6 +11,6 @@ $ python3 -m drgn --help
 
 
 if __name__ == "__main__":
-    from drgn.internal.cli import main
+    from drgn.cli import _main
 
-    main()
+    _main()
