@@ -10,9 +10,13 @@ Linux kernel networking subsystem.
 """
 
 import operator
-from typing import Iterator, Union
+from typing import Iterator, Optional, Union
 
-from drgn import NULL, IntegerLike, Object, Program, cast, container_of
+from drgn import NULL, IntegerLike, Object, Program, Type, cast, container_of, sizeof
+from drgn.helpers.common.prog import (
+    takes_object_or_program_or_default,
+    takes_program_or_default,
+)
 from drgn.helpers.linux.fs import fget
 from drgn.helpers.linux.list import hlist_for_each_entry, list_for_each_entry
 from drgn.helpers.linux.list_nulls import hlist_nulls_for_each_entry
@@ -26,8 +30,10 @@ __all__ = (
     "netdev_for_each_tx_queue",
     "netdev_get_by_index",
     "netdev_get_by_name",
+    "netdev_priv",
     "sk_fullsock",
     "sk_nulls_for_each",
+    "skb_shinfo",
 )
 
 
@@ -59,16 +65,16 @@ def SOCK_INODE(sock: Object) -> Object:
     return container_of(sock, "struct socket_alloc", "socket").vfs_inode.address_of_()
 
 
+@takes_program_or_default
 def for_each_net(prog: Program) -> Iterator[Object]:
     """
     Iterate over all network namespaces in the system.
 
     :return: Iterator of ``struct net *`` objects.
     """
-    for net in list_for_each_entry(
+    return list_for_each_entry(
         "struct net", prog["net_namespace_list"].address_of_(), "list"
-    ):
-        yield net
+    )
 
 
 _CLONE_NEWNET = 0x40000000
@@ -122,45 +128,44 @@ _NETDEV_HASHBITS = 8
 _NETDEV_HASHENTRIES = 1 << _NETDEV_HASHBITS
 
 
+@takes_object_or_program_or_default
 def netdev_get_by_index(
-    prog_or_net: Union[Program, Object], ifindex: IntegerLike
+    prog: Program, net: Optional[Object], ifindex: IntegerLike
 ) -> Object:
     """
     Get the network device with the given interface index number.
 
-    :param prog_or_net: ``struct net *`` containing the device, or
-        :class:`Program` to use the initial network namespace.
+    :param net: ``struct net *``. Defaults to the initial network namespace if
+        given a :class:`~drgn.Program` or :ref:`omitted <default-program>`.
     :param ifindex: Network interface index number.
     :return: ``struct net_device *`` (``NULL`` if not found)
     """
-    if isinstance(prog_or_net, Program):
-        prog_or_net = prog_or_net["init_net"]
-    if isinstance(ifindex, Object):
-        ifindex = ifindex.read_()
+    if net is None:
+        net = prog["init_net"]
+    ifindex = operator.index(ifindex)
 
-    head = prog_or_net.dev_index_head[
-        operator.index(ifindex) & (_NETDEV_HASHENTRIES - 1)
-    ]
+    head = net.dev_index_head[ifindex & (_NETDEV_HASHENTRIES - 1)]
     for netdev in hlist_for_each_entry("struct net_device", head, "index_hlist"):
-        if netdev.ifindex == ifindex:
+        if netdev.ifindex.value_() == ifindex:
             return netdev
 
-    return NULL(prog_or_net.prog_, "struct net_device *")
+    return NULL(prog, "struct net_device *")
 
 
+@takes_object_or_program_or_default
 def netdev_get_by_name(
-    prog_or_net: Union[Program, Object], name: Union[str, bytes]
+    prog: Program, net: Optional[Object], name: Union[str, bytes]
 ) -> Object:
     """
     Get the network device with the given interface name.
 
-    :param prog_or_net: ``struct net *`` containing the device, or
-        :class:`Program` to use the initial network namespace.
+    :param net: ``struct net *``. Defaults to the initial network namespace if
+        given a :class:`~drgn.Program` or :ref:`omitted <default-program>`.
     :param name: Network interface name.
     :return: ``struct net_device *`` (``NULL`` if not found)
     """
-    if isinstance(prog_or_net, Program):
-        prog_or_net = prog_or_net["init_net"]
+    if net is None:
+        net = prog["init_net"]
     if isinstance(name, str):
         name = name.encode()
 
@@ -169,16 +174,16 @@ def netdev_get_by_name(
     # struct netdev_name_node entries. Before that, it contained the struct
     # net_device directly.
     try:
-        entry_type = prog_or_net.prog_.type("struct netdev_name_node")
+        entry_type = prog.type("struct netdev_name_node")
         member = "hlist"
         entry_is_name_node = True
     except LookupError:
-        entry_type = prog_or_net.prog_.type("struct net_device")
+        entry_type = prog.type("struct net_device")
         member = "name_hlist"
         entry_is_name_node = False
 
     for i in range(_NETDEV_HASHENTRIES):
-        head = prog_or_net.dev_name_head[i]
+        head = net.dev_name_head[i]
         for entry in hlist_for_each_entry(entry_type, head, member):
             if entry.name.string_() == name:
                 if entry_is_name_node:
@@ -186,7 +191,33 @@ def netdev_get_by_name(
                 else:
                     return entry
 
-    return NULL(prog_or_net.prog_, "struct net_device *")
+    return NULL(prog, "struct net_device *")
+
+
+def netdev_priv(dev: Object, type: Union[str, Type] = "void") -> Object:
+    """
+    Return the private data of a network device.
+
+    >>> dev = netdev_get_by_name("wlp0s20f3")
+    >>> netdev_priv(dev)
+    (void *)0xffff9419c9dec9c0
+    >>> netdev_priv(dev, "struct ieee80211_sub_if_data")
+    *(struct ieee80211_sub_if_data *)0xffff9419c9dec9c0 = {
+        ...
+    }
+
+    :param dev: ``struct net_device *``
+    :param type: Type of private data.
+    :return: ``type *``
+    """
+    prog = dev.prog_
+    try:
+        offset = prog.cache["net_device_aligned_size"]
+    except KeyError:
+        # 31 is NETDEV_ALIGN - 1
+        offset = (sizeof(prog.type("struct net_device")) + 31) & ~31
+        prog.cache["net_device_aligned_size"] = offset
+    return Object(prog, prog.pointer_type(prog.type(type)), dev.value_() + offset)
 
 
 def sk_fullsock(sk: Object) -> bool:
@@ -209,7 +240,23 @@ def sk_nulls_for_each(head: Object) -> Iterator[Object]:
     :param head: ``struct hlist_nulls_head *``
     :return: Iterator of ``struct sock *`` objects.
     """
-    for sk in hlist_nulls_for_each_entry(
-        "struct sock", head, "__sk_common.skc_nulls_node"
-    ):
-        yield sk
+    return hlist_nulls_for_each_entry("struct sock", head, "__sk_common.skc_nulls_node")
+
+
+def skb_shinfo(skb: Object) -> Object:
+    """
+    Get the shared info for a socket buffer.
+
+    :param skb: ``struct sk_buff *``
+    :return: ``struct skb_shared_info *``
+    """
+    prog = skb.prog_
+    try:
+        NET_SKBUFF_DATA_USES_OFFSET = prog.cache["NET_SKBUFF_DATA_USES_OFFSET"]
+    except KeyError:
+        NET_SKBUFF_DATA_USES_OFFSET = sizeof(prog.type("long")) > 4
+        prog.cache["NET_SKBUFF_DATA_USES_OFFSET"] = NET_SKBUFF_DATA_USES_OFFSET
+    if NET_SKBUFF_DATA_USES_OFFSET:
+        return cast("struct skb_shared_info *", skb.head + skb.end)
+    else:
+        return cast("struct skb_shared_info *", skb.end)
