@@ -21,6 +21,7 @@
 #include "debug_info.h"
 #include "drgn_internal.h"
 #include "elf_file.h"
+#include "elf_notes.h"
 #include "error.h"
 #include "hash_table.h"
 #include "helpers.h"
@@ -486,6 +487,7 @@ static struct drgn_error *
 kernel_module_iterator_next(struct kernel_module_iterator *it)
 {
 	struct drgn_error *err;
+	struct drgn_program *prog = drgn_object_program(&it->mod);
 
 	uint64_t addr;
 	err = drgn_object_read_unsigned(&it->node, &addr);
@@ -518,24 +520,28 @@ kernel_module_iterator_next(struct kernel_module_iterator *it)
 	// Set tmp1 to the module base address and tmp2 to the size.
 	err = drgn_object_member(&it->tmp1, &it->mod, "mem");
 	if (!err) {
-		union drgn_value mod_mem_type;
-
 		// Since Linux kernel commit ac3b43283923 ("module: replace
 		// module_layout with module_memory") (in v6.4), the base and
 		// size are in the `struct module_memory mem[MOD_TEXT]` member
 		// of `struct module`.
-		err = drgn_program_find_object(drgn_object_program(&it->mod),
-					       "MOD_TEXT", NULL,
-					       DRGN_FIND_OBJECT_CONSTANT,
-					       &it->tmp2);
-		if (err)
-			return err;
-		err = drgn_object_read_integer(&it->tmp2, &mod_mem_type);
-		if (err)
-			return err;
+		if (!prog->mod_text_cached) {
+			err = drgn_program_find_object(drgn_object_program(&it->mod),
+						       "MOD_TEXT", NULL,
+						       DRGN_FIND_OBJECT_CONSTANT,
+						       &it->tmp2);
+			if (err)
+				return err;
+			union drgn_value mod_text_value;
+			err = drgn_object_read_integer(&it->tmp2,
+						       &mod_text_value);
+			if (err)
+				return err;
+			prog->mod_text = mod_text_value.uvalue;
+			prog->mod_text_cached = true;
+		}
 
 		err = drgn_object_subscript(&it->tmp1, &it->tmp1,
-					    mod_mem_type.uvalue);
+					    prog->mod_text);
 		if (err)
 			return err;
 		err = drgn_object_member(&it->tmp2, &it->tmp1, "size");
@@ -598,25 +604,6 @@ kernel_module_iterator_next(struct kernel_module_iterator *it)
 	return NULL;
 }
 
-static size_t parse_gnu_build_id_from_note(const void *note, size_t note_size,
-					   bool bswap, const void **ret)
-{
-	Elf32_Nhdr nhdr;
-	const char *name;
-	const void *desc;
-	while (next_elf_note(&note, &note_size, 4, bswap, &nhdr, &name, &desc)) {
-		if (nhdr.n_namesz == sizeof("GNU") &&
-		    memcmp(name, "GNU", sizeof("GNU")) == 0 &&
-		    nhdr.n_type == NT_GNU_BUILD_ID &&
-		    nhdr.n_descsz > 0) {
-			*ret = desc;
-			return nhdr.n_descsz;
-		}
-	}
-	*ret = NULL;
-	return 0;
-}
-
 static struct drgn_error *
 kernel_module_iterator_gnu_build_id_live(struct kernel_module_iterator *it,
 					 const void **build_id_ret,
@@ -671,8 +658,8 @@ kernel_module_iterator_gnu_build_id_live(struct kernel_module_iterator *it,
 		close(fd);
 
 		*build_id_len_ret =
-			parse_gnu_build_id_from_note(it->build_id_buf, r, false,
-						     build_id_ret);
+			parse_gnu_build_id_from_notes(it->build_id_buf, r, 4,
+						      false, build_id_ret);
 		if (*build_id_len_ret) {
 			err = NULL;
 			goto out;
@@ -764,8 +751,8 @@ kernel_module_iterator_gnu_build_id(struct kernel_module_iterator *it,
 			return err;
 
 		*build_id_len_ret =
-			parse_gnu_build_id_from_note(it->build_id_buf, size,
-						     bswap, build_id_ret);
+			parse_gnu_build_id_from_notes(it->build_id_buf, size, 4,
+						      bswap, build_id_ret);
 		if (*build_id_len_ret)
 			return NULL;
 	}
@@ -1573,7 +1560,7 @@ report_kernel_modules(struct drgn_debug_info_load_state *load,
 		struct kernel_module_file *kmod = &kmods[i];
 
 		ssize_t build_id_len =
-			dwelf_elf_gnu_build_id(kmod->elf, &kmod->gnu_build_id);
+			drgn_elf_gnu_build_id(kmod->elf, &kmod->gnu_build_id);
 		if (build_id_len < 0) {
 			err = drgn_debug_info_report_error(load, kmod->path,
 							   NULL,
