@@ -15,7 +15,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "drgn.h"
+#include "drgn_internal.h"
+#include "handler.h"
 #include "type.h"
 
 /**
@@ -103,6 +104,16 @@ struct drgn_operand_type {
 	uint64_t bit_field_size;
 };
 
+/** Convert a @ref drgn_operand_type to a @ref drgn_qualified_type. */
+static inline struct drgn_qualified_type
+drgn_operand_type_qualified(const struct drgn_operand_type *type)
+{
+	return (struct drgn_qualified_type){
+		.type = type->type,
+		.qualifiers = type->qualifiers,
+	};
+}
+
 /** Get the @ref drgn_operand_type of a @ref drgn_object. */
 static inline struct drgn_operand_type
 drgn_object_operand_type(const struct drgn_object *obj)
@@ -186,6 +197,23 @@ drgn_object_set_reference_internal(struct drgn_object *res,
 				   const struct drgn_object_type *type,
 				   uint64_t address, uint64_t bit_offset);
 
+/**
+ * Like @ref drgn_object_set_absent() but @ref drgn_object_type() was already
+ * called.
+ */
+static inline void
+drgn_object_set_absent_internal(struct drgn_object *res,
+				const struct drgn_object_type *type)
+{
+	drgn_object_reinit(res, type, DRGN_OBJECT_ABSENT);
+}
+
+struct drgn_error *
+drgn_object_slice_internal(struct drgn_object *res,
+			   const struct drgn_object *obj,
+			   const struct drgn_object_type *type,
+			   uint64_t bit_offset, uint64_t bit_field_size);
+
 /**
  * Binary operator implementation.
  *
@@ -337,7 +365,7 @@ drgn_unary_op_impl drgn_op_not_impl;
  * address of @p obj is used.
  */
 struct drgn_error *drgn_op_cast(struct drgn_object *res,
-				struct drgn_qualified_type qualified_type,
+				const struct drgn_object_type *type,
 				const struct drgn_object *obj,
 				const struct drgn_operand_type *obj_type);
 
