@@ -138,66 +138,102 @@ struct drgn_program {
 	 * from NT_ARM_PAC_MASK or VMCOREINFO.
 	 */
 	uint64_t aarch64_insn_pac_mask;
-	bool core_dump_notes_cached;
-	bool prefer_orc_unwinder;
-	const char *core_dump_fname_cached;
+	bool core_dump_threads_cached;
 
-	/*
-	 * Linux kernel-specific.
-	 */
-	/* The important parts of the VMCOREINFO note of a Linux kernel core. */
-	struct {
-		/** <tt>uname -r</tt> */
-		char osrelease[128];
-		/** PAGE_SIZE of the kernel. */
-		uint64_t page_size;
-		/**
-		 * The offset from the compiled address of the kernel image to its
-		 * actual address in memory.
-		 *
-		 * This is non-zero if kernel address space layout randomization (KASLR)
-		 * is enabled.
+	union {
+		/*
+		 * Userspace-specific.
 		 */
-		uint64_t kaslr_offset;
-		/** Kernel page table. */
-		uint64_t swapper_pg_dir;
-		/** Length of mem_section array (i.e., NR_SECTION_ROOTS). */
-		uint64_t mem_section_length;
-		/** VA_BITS on AArch64. */
-		uint64_t va_bits;
-		/** TCR_EL1_T1SZ on AArch64. */
-		uint64_t tcr_el1_t1sz;
-		/** phys_base on x86_64 */
-		uint64_t phys_base;
-		/** Whether 5-level paging was enabled on x86-64. */
-		bool pgtable_l5_enabled;
-		/** Whether LPAE was enabled on Arm. */
-		bool arm_lpae;
-		/** Whether CRASHTIME was in the VMCOREINFO. */
-		bool have_crashtime;
-		/** PAGE_SHIFT of the kernel (derived from PAGE_SIZE). */
-		int page_shift;
+		struct {
+			/** Cached `pr_fname` from `NT_PRPSINFO` note. */
+			const char *core_dump_fname_cached;
+		};
 
-		/** The original vmcoreinfo data, to expose as an object */
-		char *raw;
-		size_t raw_size;
-	} vmcoreinfo;
+		/*
+		 * Linux kernel-specific.
+		 */
+		struct {
+			/*
+			 * Important parts of the VMCOREINFO note of a Linux
+			 * kernel core.
+			 */
+			struct {
+				/** `uname -r` */
+				char osrelease[128];
+				/** `PAGE_SIZE` of the kernel. */
+				uint64_t page_size;
+				/**
+				 * The offset from the compiled address of the
+				 * kernel image to its actual address in memory.
+				 *
+				 * This is non-zero if kernel address space
+				 * layout randomization (KASLR) is enabled.
+				 */
+				uint64_t kaslr_offset;
+				/** Kernel page table. */
+				uint64_t swapper_pg_dir;
+				/**
+				 * Length of mem_section array (i.e.,
+				 * `NR_SECTION_ROOTS`).
+				 */
+				uint64_t mem_section_length;
+				/** `VA_BITS` on AArch64. */
+				uint64_t va_bits;
+				/** `TCR_EL1_T1SZ` on AArch64. */
+				uint64_t tcr_el1_t1sz;
+				/** `phys_base` on x86_64 */
+				uint64_t phys_base;
+				/**
+				 * Whether 5-level paging was enabled on x86-64.
+				 */
+				bool pgtable_l5_enabled;
+				/** Whether LPAE was enabled on Arm. */
+				bool arm_lpae;
+				/** Whether `CRASHTIME` was in the VMCOREINFO. */
+				bool have_crashtime;
+				/** Whether `phys_base` was in the VMCOREINFO. */
+				bool have_phys_base;
+				/**
+				 * `PAGE_SHIFT` of the kernel (derived from
+				 * `PAGE_SIZE`).
+				 */
+				int page_shift;
+
+				/** The original vmcoreinfo data, to expose as an object */
+				char *raw;
+				size_t raw_size;
+			} vmcoreinfo;
+			/*
+			 * Difference between a virtual address in the direct
+			 * mapping and the physical address it maps to.
+			 */
+			uint64_t direct_mapping_offset;
+			/** Cached value of `MOD_TEXT` in the kernel. */
+			uint64_t mod_text;
+			/*
+			 * Whether @ref drgn_program::direct_mapping_offset has
+			 * been cached.
+			 */
+			bool direct_mapping_offset_cached;
+			/**
+			 * Whether @ref drgn_program::mod_text has been cached.
+			 */
+			bool mod_text_cached;
+			/*
+			 * Whether we are currently in address translation. Used
+			 * to prevent address translation from recursing.
+			 */
+			bool in_address_translation;
+		};
+	};
 	/*
-	 * Difference between a virtual address in the direct mapping and the
-	 * physical address it maps to.
+	 * Linux kernel-specific, but simplifies init/deinit to have them
+	 * outside of the union above.
 	 */
-	uint64_t direct_mapping_offset;
 	/* Cached vmemmap. */
 	struct drgn_object vmemmap;
 	/* Page table iterator. */
 	struct pgtable_iterator *pgtable_it;
-	/*
-	 * Whether we are currently in address translation. Used to prevent
-	 * address translation from recursing.
-	 */
-	bool in_address_translation;
-	/* Whether @ref drgn_program::direct_mapping_offset has been cached. */
-	bool direct_mapping_offset_cached;
 
 	/*
 	 * Logging.
@@ -253,6 +289,28 @@ struct drgn_error *drgn_program_init_kernel(struct drgn_program *prog);
  */
 struct drgn_error *drgn_program_init_pid(struct drgn_program *prog, pid_t pid);
 
+/**
+ * Return whether a @ref drgn_program is a userspace process running on the
+ * local machine.
+ */
+static inline bool
+drgn_program_is_userspace_process(struct drgn_program *prog)
+{
+	return (prog->flags & (DRGN_PROGRAM_IS_LINUX_KERNEL
+			       | DRGN_PROGRAM_IS_LIVE
+			       | DRGN_PROGRAM_IS_LOCAL))
+	       == (DRGN_PROGRAM_IS_LIVE | DRGN_PROGRAM_IS_LOCAL);
+}
+
+/** Return whether a @ref drgn_program is a core dump of a userspace process. */
+static inline bool
+drgn_program_is_userspace_core(struct drgn_program *prog)
+{
+	return (prog->flags &
+		(DRGN_PROGRAM_IS_LINUX_KERNEL | DRGN_PROGRAM_IS_LIVE)) == 0
+	       && prog->core;
+}
+
 static inline struct drgn_error *
 drgn_program_is_little_endian(struct drgn_program *prog, bool *ret)
 {
