@@ -18,6 +18,7 @@
 #include "../hash_table.h"
 #include "../pp.h"
 #include "../program.h"
+#include "../symbol.h"
 
 /* These were added in Python 3.7. */
 #ifndef Py_UNREACHABLE
@@ -39,6 +40,10 @@
     } while (0)
 #endif
 
+#if PY_VERSION_HEX < 0x030d00a1
+#define PyThreadState_GetUnchecked _PyThreadState_UncheckedGet
+#endif
+
 #define DRGNPY_PUBLIC __attribute__((__visibility__("default")))
 
 // PyLong_From* and PyLong_As* for stdint.h types. These use _Generic for
@@ -108,6 +113,11 @@ typedef struct {
 	PyObject *attr_cache;
 } DrgnType;
 
+typedef struct {
+	PyObject_HEAD
+	struct drgn_symbol_index index;
+} SymbolIndex;
+
 typedef struct {
 	PyObject_HEAD
 	/*
@@ -242,6 +252,7 @@ extern PyTypeObject Register_type;
 extern PyTypeObject StackFrame_type;
 extern PyTypeObject StackTrace_type;
 extern PyTypeObject Symbol_type;
+extern PyTypeObject SymbolIndex_type;
 extern PyTypeObject Thread_type;
 extern PyTypeObject ThreadIterator_type;
 extern PyTypeObject TypeEnumerator_type;
@@ -304,6 +315,8 @@ Program *program_from_kernel(PyObject *self);
 Program *program_from_pid(PyObject *self, PyObject *args, PyObject *kwds);
 
 PyObject *Symbol_wrap(struct drgn_symbol *sym, PyObject *name_obj);
+PyObject *Symbol_list_wrap(struct drgn_symbol **symbols, size_t count,
+			   PyObject *name_obj);
 
 PyObject *Thread_wrap(struct drgn_thread *drgn_thread);
 
@@ -344,6 +357,8 @@ struct index_arg {
 };
 int index_converter(PyObject *o, void *p);
 
+int u64_converter(PyObject *o, void *p);
+
 struct path_arg {
 	bool allow_fd;
 	bool allow_none;
@@ -356,6 +371,10 @@ struct path_arg {
 int path_converter(PyObject *o, void *p);
 void path_cleanup(struct path_arg *path);
 
+#define PATH_ARG(name, ...)				\
+	__attribute__((__cleanup__(path_cleanup)))	\
+	struct path_arg name = { __VA_ARGS__ }
+
 struct enum_arg {
 	PyObject *type;
 	unsigned long value;
@@ -387,5 +406,9 @@ DrgnObject *drgnpy_linux_helper_pid_task(PyObject *self, PyObject *args,
 DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args);
 PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *arg);
 PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *arg);
+PyObject *drgnpy_linux_helper_load_proc_kallsyms(PyObject *self, PyObject *args,
+						 PyObject *kwds);
+PyObject *drgnpy_linux_helper_load_builtin_kallsyms(PyObject *self, PyObject *args,
+						    PyObject *kwds);
 
 #endif /* DRGNPY_H */
