@@ -14,6 +14,7 @@
 #include "dwarf_constants.h"
 #include "dwarf_info.h"
 #include "elf_file.h"
+#include "elf_notes.h"
 #include "error.h"
 #include "helpers.h"
 #include "minmax.h"
@@ -650,7 +651,7 @@ drgn_get_initial_registers_from_kernel_core_dump(struct drgn_program *prog,
 		const void *p = drgn_object_buffer(&tmp);
 		size_t size = drgn_object_size(&tmp);
 		bool bswap = drgn_platform_bswap(&prog->platform);
-		Elf32_Nhdr nhdr;
+		GElf_Nhdr nhdr;
 		const char *name;
 		const void *desc;
 		while (next_elf_note(&p, &size, 4, bswap, &nhdr, &name, &desc)
@@ -1150,10 +1151,13 @@ static struct drgn_error *drgn_get_stack_trace(struct drgn_program *prog,
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "cannot unwind stack without platform");
 	}
-	if ((prog->flags & (DRGN_PROGRAM_IS_LINUX_KERNEL |
-			    DRGN_PROGRAM_IS_LIVE)) == DRGN_PROGRAM_IS_LIVE) {
+	if (drgn_program_is_userspace_process(prog)) {
 		return drgn_error_create(DRGN_ERROR_NOT_IMPLEMENTED,
 					 "stack unwinding is not yet supported for live processes");
+	} else if (!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		   && !drgn_program_is_userspace_core(prog)) {
+		return drgn_error_create(DRGN_ERROR_NOT_IMPLEMENTED,
+					 "stack unwinding is not supported for this program");
 	}
 
 	size_t trace_capacity = 1;
