@@ -11,7 +11,6 @@ from drgn.helpers.linux.mm import pfn_to_virt
 from tests.linux_kernel import (
     HAVE_FULL_MM_SUPPORT,
     LinuxKernelTestCase,
-    fork_and_stop,
     skip_unless_have_full_mm_support,
     skip_unless_have_stack_tracing,
     skip_unless_have_test_kmod,
@@ -52,20 +51,28 @@ class TestIdentifyAddress(LinuxKernelTestCase):
 
     @skip_unless_have_test_kmod
     def test_identify_vmap(self):
-        self.assertTrue(
-            identify_address(self.prog["drgn_test_vmalloc_va"]).startswith("vmap: 0x")
-        )
+        for cache in (None, {}):
+            with self.subTest("uncached" if cache is None else "cached"):
+                self.assertTrue(
+                    identify_address(
+                        self.prog["drgn_test_vmalloc_va"], cache=cache
+                    ).startswith("vmap: 0x")
+                )
 
     @skip_unless_have_test_kmod
     def test_identify_vmap_stack(self):
         if not self.prog["drgn_test_vmap_stack_enabled"]:
             self.skipTest("kernel does not use vmap stacks (CONFIG_VMAP_STACK)")
-        self.assertEqual(
-            identify_address(
-                self.prog, self.prog["drgn_test_kthread"].stack.value_() + 1234
-            ),
-            f"vmap stack: {self.prog['drgn_test_kthread'].pid.value_()} (drgn_test_kthre) +0x4d2",
-        )
+        for cache in (None, {}):
+            with self.subTest("uncached" if cache is None else "cached"):
+                self.assertEqual(
+                    identify_address(
+                        self.prog,
+                        self.prog["drgn_test_kthread"].stack.value_() + 1234,
+                        cache=cache,
+                    ),
+                    f"vmap stack: {self.prog['drgn_test_kthread'].pid.value_()} (drgn_test_kthre) +0x4d2",
+                )
 
     @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
@@ -92,23 +99,26 @@ class TestPrintAnnotatedMemory(LinuxKernelTestCase):
                 self.prog["drgn_test_small_slab_objects"].address_,
                 sizeof(self.prog["drgn_test_small_slab_objects"]),
             )
-        self.assertIn("slab object: drgn_test_small+0x0", f.getvalue())
+        # For CONFIG_SLOB, we cannot find slab objects. However,
+        # print_annotated_memory() should still function with no error. So we
+        # don't skip the test here: just skip the assertion.
+        if not self.prog["drgn_test_slob"]:
+            self.assertIn("slab object: drgn_test_small+0x0", f.getvalue())
 
 
 class TestPrintAnnotatedStack(LinuxKernelTestCase):
     @skip_unless_have_stack_tracing
     @skip_unless_have_test_kmod
     def test_print_annotated_stack(self):
-        with fork_and_stop() as pid:
-            trace = self.prog.stack_trace(pid)
+        trace = self.prog.stack_trace(self.prog["drgn_test_kthread"])
 
-            f = io.StringIO()
-            with redirect_stdout(f):
-                print_annotated_stack(trace)
+        f = io.StringIO()
+        with redirect_stdout(f):
+            print_annotated_stack(trace)
 
-            printed_trace = f.getvalue()
+        printed_trace = f.getvalue()
 
-            if HAVE_FULL_MM_SUPPORT and not self.prog["drgn_test_slob"]:
-                self.assertIn("slab object: task_struct", printed_trace)
-            self.assertIn("[function symbol: schedule", printed_trace)
-            self.assertIn("schedule at ", printed_trace)
+        if HAVE_FULL_MM_SUPPORT and not self.prog["drgn_test_slob"]:
+            self.assertIn("slab object: drgn_test_small", printed_trace)
+        self.assertIn("[function symbol: schedule", printed_trace)
+        self.assertIn("schedule at ", printed_trace)
