@@ -51,6 +51,10 @@ _PATCHES = (
         name="proc-kcore-allow-enabling-CONFIG_PROC_KCORE-on-ARM.patch",
         versions=((None, None),),
     ),
+    _Patch(
+        name="9p-fix-slab-cache-name-creation-for-real.patch",
+        versions=((KernelVersion("6.12"), None),),
+    ),
     _Patch(
         name="filelock-fix-name-of-file_lease-slab-cache.patch",
         versions=((KernelVersion("6.9"), KernelVersion("6.10")),),
@@ -138,6 +142,10 @@ _PATCHES = (
         name="s390-crash-fix-proc-vmcore-reads.patch",
         versions=((KernelVersion("5.18"), KernelVersion("6.0")),),
     ),
+    _Patch(
+        name="lib-raid6-add-option-to-skip-algo-benchmarking.patch",
+        versions=((None, KernelVersion("5.0")),),
+    ),
 )
 
 
